#! /usr/bin/env python

import os, sys, subprocess, argparse

parser = argparse.ArgumentParser()
parser.add_argument('-t', '--type', default='Release', 
                    choices=['Debug', 'Release', 'RelWithDebInfo'])
parser.add_argument('-c', '--compiler', default='gnu', 
                    choices=['gnu', 'intel'])
args = parser.parse_args()

myenv = os.environ.copy()

if args.compiler == 'gnu':
   myenv['CC'] = 'gcc'
   myenv['CXX'] = 'g++'
elif args.compiler == 'intel':
   myenv['CC'] = 'icx'
   myenv['CXX'] = 'icpx'

compileCmd = ['make', '--no-print-directory', 'install']

base = os.getcwd()
srcDir = os.path.join(base, 'src')
buildDir = os.path.join(base, 'build', args.compiler)
installDir = os.path.join(base, 'install', args.compiler)

cmake_params = ['-DCMAKE_BUILD_TYPE=' + args.type]
cmake_params.append('-DCMAKE_INSTALL_PREFIX=' + installDir)

if not os.path.exists(buildDir):
  os.makedirs(buildDir)

configureCmd = ['cmake'] + cmake_params + [srcDir]
print(' '.join(configureCmd))
err = subprocess.call(configureCmd, cwd=buildDir, env=myenv)

if err == 0:
  err = subprocess.call(compileCmd, cwd=buildDir, env=myenv)

