#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <mpi.h>
#include "config.h"

int main (int argc, char* argv[])
{
    int rank, size;
    double pi=0.0, result=0.0, sum=0.0, begin=0.0, end=0.0, x2;
    double d, d2;

    long long i, N = argc > 1 ? atoll(argv[1]) : defaultN;
    d = 1.0/N;
    d2 = d*d;

    //MPI_Init (&argc, &argv);

    int required = MPI_THREAD_FUNNELED, provided;
    MPI_Init_thread (&argc, &argv, required, &provided);
    if (provided < required)
    {
       printf("erreur");
       exit(-1);
    }
    MPI_Comm_rank (MPI_COMM_WORLD, &rank);
    MPI_Comm_size (MPI_COMM_WORLD, &size);
    
    MPI_Barrier(MPI_COMM_WORLD);
    begin = MPI_Wtime();
    
    #pragma omp parallel for private(x2) reduction(+:result)
    for (i=rank; i<N; i+=size)
    {
        x2=d2*i*i;
        result+=1.0/(1.0+x2);
    }
    
    MPI_Reduce(&result, &sum, 1, MPI_DOUBLE, MPI_SUM, 0, MPI_COMM_WORLD);
    
    MPI_Barrier(MPI_COMM_WORLD);
    end = MPI_Wtime();
    
    if (rank==0)
    {
        pi=4*d*sum;
        printf("np=%2d;\nTime       =      %9.3f s\n\nPI         =%15.10g\nPI(machine)=%15.10g\nError      =%15.2g\n", 
               size, end-begin, pi, 4*atan(1.0), fabs(4*atan(1.0)-pi));
    }
    
    MPI_Finalize();
    
    return 0;
}
