#! /usr/bin/env python
# -*- coding: utf-8 -*-

try:
    unicode('')
except NameError:
    unicode = str

import os, sys, subprocess, argparse
from util import *


def manage(args):
    job = '''#!/bin/bash

#SBATCH --job-name=build_pi
#SBATCH --output=output_build.txt
#SBATCH --error=output_build.txt

@ACC@

#SBATCH --ntasks=1
#SBATCH --cpus-per-task=1
#SBATCH --time=@RUNTIME@

## load modules

module purge
module load cmake/3.19.7

@MODULES@

## execution
./build.py -c ${COMPILER}

'''

    s = job
    s = s.replace("@ACC@", queue())

    rs = 0
    rm = 2
    rh = 0
    while rm > 60:
        rh += 1
        rm -= 60
    r = "{:02d}:{:02d}:00".format(rh, rm)
    s = s.replace("@RUNTIME@", r)

    if args.compiler == 'gnu':
       m = '''
module load gcc/10.2.0
module load openmpi/4.1.4

export CC=gcc
export CXX=g++
COMPILER=gnu
'''
    else:
       m = '''
module load intel_compiler/oneapi_2023.2.0
module load intel_mpi/2021.9.0
export I_MPI_CXX=icpx
export I_MPI_CC=icx
export CC=icx
export CXX=icpx
COMPILER=intel
'''

    s = s.replace("@MODULES@", m)

    jobFile = 'job_build.sh'
    with open(jobFile, 'w') as f:
       f.write(s)
 
    subprocess.call(['sbatch', jobFile])

if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('-c', '--compiler', choices = ['gnu', 'intel'], default='gnu')
    args = parser.parse_args()
    manage(args)
