#! /usr/bin/env python
# -*- coding: utf-8 -*-

try:
    unicode('')
except NameError:
    unicode = str

import os, sys, subprocess
from util import *

def manage(args):
  installDir = os.path.join('install', args.compiler)

  cases = [
    [ 
      'seq',        
      os.path.join(installDir, 'pi'),
      '',
      1, 
      1
    ],
    [ 
      'openmp',
      os.path.join(installDir, 'openmp_pi'),
      'export OMP_NUM_THREADS=8',
      1,
      8
    ],
    [
      'mpi',
      'mpirun ' + os.path.join(installDir, 'mpi_pi'),
      '',
      8, 
      1
    ],
    [ 
      'mpi_openmp', 
      'mpirun -n $SLURM_NTASKS ' + os.path.join(installDir, 'mpi_openmp_pi'),
      'export OMP_NUM_THREADS=2',
      4,
      2 
    ]
  ]

  job = '''#!/bin/bash

#SBATCH --job-name=@VERSION@
#SBATCH --output=output_@VERSION@.txt
#SBATCH --error=output_@VERSION@.txt

@ACC@

#SBATCH --ntasks=@NPROCESS@
#SBATCH --cpus-per-task=@NTHREADS@
#SBATCH --time=@RUNTIME@

## load modules
module purge
@MODULES@

## execution
@EXPORT@
echo @EXEC@
@EXEC@

'''

  for c in cases:
    s = job
    s = s.replace("@ACC@", queue())
    s = s.replace("@VERSION@", c[0])
    s = s.replace("@EXEC@", c[1])
    s = s.replace("@EXPORT@", c[2])
    s = s.replace("@NPROCESS@", str(c[3]))
    s = s.replace("@NTHREADS@", str(c[4]))

    rs = 0
    rm = 2
    rh = 0
    while rm > 60:
      rh += 1
      rm -= 60
    r = "{:02d}:{:02d}:00".format(rh, rm)
    s = s.replace("@RUNTIME@", r)
    if args.compiler == 'gnu':
       m = '''
module load gcc/10.2.0
module load openmpi/4.1.4
COMPILER=gnu
'''
    else:
       m = '''
module load intel_compiler/oneapi_2023.2.0
module load intel_mpi/2021.9.0
COMPILER=intel
'''
    s = s.replace("@MODULES@", m)


    jobFile = 'job_' + c[0] + '.sh'
    with open(jobFile, 'w') as f:
       f.write(s)
 
    subprocess.call(['sbatch', jobFile])

if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('-c', '--compiler', choices = ['gnu', 'intel'], default='gnu')
    args = parser.parse_args()
    manage(args)
